// $Id: padma.js,v 1.3 2005/04/18 19:19:55 vnagarjuna Exp $ -->

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Nagarjuna Venna.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nagarjuna Venna <vnagarjuna@yahoo.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//Parser for RTS Text - returns syllables

function RTSParser(input) {
    this.index = 0;
    this.length = input.length;
    this.text = input;
}

function RTSParser_next()
{
    var response = "";
    var key = null;
    var curLen = 0;

    var remaining = this.length - this.index;
    for(; remaining > 0; remaining = this.length - this.index) {
        var maxLookupLen = remaining > RTS.maxLookupLen ? RTS.maxLookupLen : remaining;
        
        var value;
        var bufLen = 0;
        for(var i = maxLookupLen; i > 0; i--) {
            key = this.text.substring(this.index, this.index + i);
            value = RTS.lookup(key);
            if (value != null) {
                //found a match
                if (!RTS.isAnusvara(key)) {
                    bufLen = i;
                    break;
                }
                //the match is one of the special sunnas (n or m followed by some consonant). Treat this as sunna
                //only if this is the beginning of a syllable. Lame example: #arnku# - the nk here should not be sunna
                //This piece of code is probably not necessary, I doubt there is a word in telugu that would match this
                else if (curLen == 0) {
                    //only the n/m is consumed - so buffer length is 1 less
                    bufLen = i - 1;
                    break;
                }
            }
        }

        //alert("match for key " + key + ", value = " + value);
		
        if (value == null) {
            break;
        }

        var digit = RTS.isDigit(key);
        if (digit) {
            //we will parse the digit again the next time
            if (curLen != 0)
                break;
        }

        //transform value to the vattu or gunintam form if necessary
        if (curLen != 0)
            value = RTS.dependentForm(value);
			
        this.index += bufLen;			
        if (RTS.isVowel(key) || RTS.isSpecial(key) || digit) {
            //alert("returning " + response + value);
            return response + value;
        }
				
        response += value;
        curLen++;
    }
		
    if (curLen == 0) {
        //can happen in two cases, bad clients (of RTSParser) or if the character we read is not recognized in RTS
        //don't worry about bad clients - if next() is called even after hasNext() returns false, there is no harm in incrmenting index
        //Those not recognized in RTS are returned as is.
        this.index++;
        //alert("returning " + key);
        return key;
    }
    //act as if there is a virama at the end unless it is m, then treat it as sunna
    if (curLen == 1 && RTS.isSunna(response)) {
        //alert("returning anusvara");
        return RTS.lookupAnusvara();
    }
    //alert("returning " + response + RTS.lookupPollu());
    return response + RTS.lookupPollu();
}

